#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "xpsk31.h"


/* Main window and menu */
extern GtkWidget
  *main_window,
  *gbl_scope_label,
  *gbl_popup_menu;

/* Text buffers for text viewers */
extern GtkTextBuffer
  *gbl_rx_text_buffer,
  *gbl_tx_text_buffer;

/* Pixbuffer for waterfall */
extern GdkPixbuf *gbl_wfall_pixbuf;
extern guchar *gbl_wfall_pixels;
extern gint
  gbl_wfall_rowstride,
  gbl_wfall_n_channels,
  gbl_wfall_width,
  gbl_wfall_height;

/* For clearing text buffers */
GtkTextBuffer *text_buffer;

/* Keyboard entry buffer */
guint key_buff[32];
/* Keystrokes counter */
int key_cnt = 0;

/* Receive freq error correction factor */
int freq_error = 0;

/* Various dialogue widgets */
static GtkWidget
  *quit_dialog = NULL,
  *save_dialog = NULL,
  *error_dialog = NULL,
  *bad_entry_dialog = NULL;

/* File variables */
extern FILE
  /* File for recording QSO's */
  *qso_record_fp,
  /* File for ADIF QSO record */
  *log_adif_fp,
  /* File for station log */
  *station_log_fp;

/* QSO data record */
extern qso_record_t qso_record;

/* Runtime config data */
extern rc_data_t rc_data;

/* Index to macros */
extern int gbl_macro_idx;

/* Waterfall pixmap */
extern GdkPixbuf *gbl_wfall_pixbuf;

/* Waterfall window pixbuf */
extern gint
  gbl_wfall_rowstride,
  gbl_wfall_n_channels,
  gbl_wfall_width,
  gbl_wfall_height;



/*  Error_Dialog()
 *
 *  Opens an error dialog box
 */

  void
Error_Dialog( char *message, gboolean hide )
{
  if( error_dialog != NULL ) return;
  error_dialog = create_error_dialog();
  GtkWidget *label = lookup_widget( error_dialog, "error_message" );
  gtk_label_set_text( GTK_LABEL(label), message );
  if( hide )
	gtk_widget_hide( lookup_widget(error_dialog, "error_ok_button") );
  gtk_widget_show( error_dialog );
}


  void
Quit_Dialog( char *message )
{
  quit_dialog = create_quit_dialog();
  GtkWidget *label = lookup_widget( quit_dialog, "quit_message" );
  gtk_label_set_text( GTK_LABEL(label), message );
  gtk_widget_show( quit_dialog );
}

  void
Save_Dialog( char *message )
{
  save_dialog = create_save_dialog();
  GtkWidget *label = lookup_widget( save_dialog, "save_message" );
  gtk_label_set_text( GTK_LABEL(label), message );
  gtk_widget_show( save_dialog );
}


  void
Bad_Entry_Dialog( char *message )
{
  bad_entry_dialog = create_bad_entry_dialog();
  GtkWidget *label = lookup_widget( bad_entry_dialog, "bad_entry" );
  gtk_label_set_text( GTK_LABEL(label), message );
  gtk_widget_show( bad_entry_dialog );
}


void
on_main_window_destroy(
	GtkObject       *object,
	gpointer         user_data)
{
  Cleanup();
  gtk_main_quit();
}


gboolean
on_main_window_delete_event(
	GtkWidget   *widget,
	GdkEvent    *event,
	gpointer     user_data)
{
  if( isFlagSet(TRANSMIT_MODE) ||
	  isFlagSet(RECEIVE_MODE) )
  {
	Quit_Dialog(
		"xpsk31 is running\n"
		"Still want to quit?" );
	return TRUE;
  }
  else
	return FALSE;
}


gboolean
on_tx_textview_button_press_event(
	GtkWidget       *widget,
	GdkEventButton  *event,
	gpointer         user_data)
{
  switch( event->button )
  {
	case 2: /* Toggle Tx/Rx/Macro modes */
	  if( isFlagSet(TRANSMIT_MACRO) )
		ClearFlag( TRANSMIT_MACRO  );
	  else
		if( isFlagSet(TRANSMIT_KEYBD) )
		{
		  ClearFlag( TRANSMIT_KEYBD  );
		  SetFlag( RECEIVE_MODE );
		  gtk_idle_add( Receive_Mode, NULL );
		}
		else
		{
		  SetFlag( TRANSMIT_KEYBD );
		  ClearFlag( RECEIVE_MODE );
		  Set_TxRx_Labels();
		  gtk_idle_add( Transmit_Keybd, "dummy" );
		}
	  break;

	case 3: /* Popup menu */
	  if( isFlagSet(SOUND_CARD_SETUP) &&
		  isFlagSet(TRANSMIT_KEYBD) )
		gtk_widget_set_sensitive( lookup_widget(gbl_popup_menu,
			  "identify_in_cw"), TRUE );
	  else
		gtk_widget_set_sensitive( lookup_widget(gbl_popup_menu,
			  "identify_in_cw"), FALSE );

	  gtk_menu_popup( GTK_MENU(gbl_popup_menu),
		  NULL, NULL, NULL, NULL, 3, 0);

	  /* Needed for 'Clear Window' */
	  text_buffer = gbl_tx_text_buffer;

  } /* switch( event->button ) */

  return TRUE;
}


gboolean
on_rx_textview_button_press_event(
	GtkWidget       *widget,
	GdkEventButton  *event,
	gpointer         user_data)
{
  switch( event->button )
  {
	case 2: /* Toggle Tx/Rx/Macro modes */
	  if( isFlagSet(RECEIVE_MODE) )
	  {
		ClearFlag( RECEIVE_MODE );
		Set_TxRx_Labels();
	  }
	  else
	  {
		if( isFlagClear(TRANSMIT_MACRO) )
		{
		  SetFlag( RECEIVE_MODE );
		  ClearFlag( TRANSMIT_KEYBD );
		  gtk_idle_add( Receive_Mode, NULL );
		}
		else
		{
		  ClearFlag( TRANSMIT_MACRO );
		  SetFlag( TRANSMIT_KEYBD );
		  Set_TxRx_Labels();
		}

	  } /* if( isFlagSet(RECEIVE_MODE) ) */
	  break;

	case 3: /* Popup menu */
	  if( isFlagSet(SOUND_CARD_SETUP) &&
		  isFlagSet(TRANSMIT_KEYBD) )
		gtk_widget_set_sensitive( lookup_widget
			(gbl_popup_menu, "identify_in_cw"), TRUE );
	  else
		gtk_widget_set_sensitive( lookup_widget
			(gbl_popup_menu, "identify_in_cw"), FALSE );

	  gtk_menu_popup( GTK_MENU(gbl_popup_menu),
		  NULL, NULL, NULL, NULL, 3, 0);

	  /* Needed for 'Clear Window' */
	  text_buffer = gbl_rx_text_buffer;

  } /* switch( event->button ) */

  return TRUE;
}


void
on_squelch_value_changed(
	GtkRange   *range,
	gpointer    user_data)
{
  GtkWidget *hscale;
  GtkAdjustment *adjustment;

  /* Read and enter squelch level */
  hscale = lookup_widget( main_window, "squelch_value" );
  adjustment = gtk_range_get_adjustment( GTK_RANGE(hscale) );
  rc_data.sqlch_thr = (int)adjustment->value;
}


void
on_op_name_changed(
	GtkEditable  *editable,
	gpointer      user_data)
{
  char buff[13];
  int len, idx;

  /* Read entry, ignore blank field */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen(buff);
  if( len == 0 ) return;

  /* Capitalize letters if enabled */
  if( isFlagSet(CAPITAL_LETTERS) )
	for( idx = 0; idx < len; idx++ )
	  if( (buff[idx] > 0x60) &&
		  (buff[idx] < 0x7b) )
		buff[idx] -= 0x20;

  /* Enter data to field QSO record structure */
  strncpy( qso_record.dx_name, buff, 13 );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_name );
}


void
on_qth_name_changed(
	GtkEditable  *editable,
	gpointer      user_data)
{
  char buff[13];
  int len, idx;

  /* Read entry, ignore blank field */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen(buff);
  if( len == 0 )
	return;

  /* Capitalize letters if enabled */
  if( isFlagSet(CAPITAL_LETTERS) )
	for( idx = 0; idx < len; idx++ )
	  if( (buff[idx] > 0x60) &&
		  (buff[idx] < 0x7b) )
		buff[idx] -= 0x20;

  /* Enter data to field and QSO record structure */
  strncpy( qso_record.dx_qth, buff, 13 );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_qth );
}


void
on_locator_changed(
	GtkEditable     *editable,
	gpointer         user_data)
{
  int len, idx;
  char buff[7];

  /* Get entry field, ignore blank field */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen(buff);
  if( len == 0 ) return;

  for( idx = 0; idx < len; idx++ )
  {
	/* Capitalize letters */
	if( (buff[idx] > 0x60) &&
		(buff[idx] < 0x7b) )
	  buff[idx] -= 0x20;

	/* Validate grid locator */
	switch( idx )
	{
	  case 0: case 1: /* First letters */
		if( (buff[idx] < 'A') || (buff[idx] > 'S') )
		{
		  Bad_Entry_Dialog(
			  "Invalid character\n"
			  "Letters A-S only" );
		  return;
		}
		break;

	  case 2: case 3: /* Middle numbers */
		if( (buff[idx] < '0') || (buff[idx] > '9') )
		{
		  Bad_Entry_Dialog(
			  "Invalid character\n"
			  "Numbers 0-9 only" );
		  return;
		}
		break;

	  case 4: case 5: /* Last letters */
		if( (buff[idx] < 'A') || (buff[idx] > 'X') )
		{
		  Bad_Entry_Dialog(
			  "Invalid character\n"
			  "Letters A-X only" );
		  return;
		}

	} /* switch( idx ) */

  } /* for( idx = 0; idx < len; idx++ ) */

  /* Enter data to field and QSO record structure */
  strncpy( qso_record.dx_loc, buff, 7 );
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_loc );
}


void
on_band_changed(
	GtkEditable     *editable,
	gpointer         user_data)
{
  int len, idx;
  char buff[14];

  qso_record.freq[0] = '\0';

  /* Get entry field, ignore blank field */
  strcpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)) );
  len = strlen(buff);
  if( len == 0 )
	return;

  /* Reject non-numbers except '.' and space */
  for( idx = 0; idx < len; idx++ )
	if( ((buff[idx]  < '0') || (buff[idx]  > '9')) &&
		((buff[idx] != '.') && (buff[idx] != ' ')) )
	{
	  Bad_Entry_Dialog(
		  "Invalid character entered\n"
		  "Numbers and '.' only allowed" );
	  return;
	}

  /* Enter data to QSO record structure */
  strncpy( qso_record.freq, buff, 14 );
}


void
on_callsign_changed(
	GtkEditable     *editable,
	gpointer         user_data)
{
  int len, idx, idy;
  char buff[15];

  /* Get entry field, ignore blank field */
  strncpy( buff, gtk_entry_get_text(GTK_ENTRY(editable)), 15 );
  len = strlen( buff );
  if( len == 0 )
	return;

  for( idx = 0; idx < len; idx++ )
  {
	/* Capitalize letters */
	if( (buff[idx] > 0x60) &&
		(buff[idx] < 0x7b) )
	  buff[idx] -= 0x20;

	/* Allow only alpha-numerics and '/' */
	if( ((buff[idx]  < 'A') || (buff[idx]  > 'Z')) &&
		((buff[idx]  < '0') || (buff[idx]  > '9')) &&
		((buff[idx] != '/') && (buff[idx] != ' ')) )
	{
	  Bad_Entry_Dialog(
		  "Invalid character entered\n"
		  "Alpha-numerics and '/' only" );
	  return;
	}

  } /* for( idx = 0; idx < len; idx++ ) */

  /* Enter data to field and QSO record structure */
  idy = 0;
  for( idx = 0; idx < len; idx++ )
	if( buff[idx] != ' ' ) /* ignore space */
	  qso_record.dx_call[idy++] = buff[idx];
  qso_record.dx_call[idy] = '\0';
  gtk_entry_set_text( GTK_ENTRY(editable), qso_record.dx_call );

  SetFlag( SAVE_RECORD );
}


void
on_new_record_clicked(
	GtkButton   *button,
	gpointer     user_data)
{
  /* Variables for time and date */
  time_t tp;    /* Time type     */
  struct tm dt; /* Date and time */

  /* For reading Tcvr status */
  char
	mode,
	freq[8],
	rst[4];

  /* Save record if needed and completed */
  if( isFlagSet(SAVE_RECORD) )
  {
	if( !Read_QSO_Record() )
	{
	  Bad_Entry_Dialog(
		  "QSO Record is not complete\n"
		  "Cannot save and open new Record" );
	  return;
	}
	else
	  Save_QSO_Record();
  }

  /* Initialize for new entry */
  Clear_Record_Fields( FALSE );

  /* Read Receiver status if CAT enabled */
  if( isFlagSet(ENABLE_CAT) )
  {
	GtkWidget *entry;

	Read_Rx_Status( &mode, freq, rst );
	if( isFlagSet(SERIAL_IO_ERROR) )
	  return;
	entry = lookup_widget( main_window, "rst_out" );
	gtk_entry_set_text( GTK_ENTRY(entry), rst );
	entry = lookup_widget( main_window, "band" );
	gtk_entry_set_text( GTK_ENTRY(entry), freq );
  }

  /* Enter time and date to QSO record */
  time( &tp );
  dt = *gmtime( &tp );
  strftime( qso_record.date,     12, "%d/%b/%Y", &dt );
  strftime( qso_record.time,      6, "%H:%M",    &dt );
  strftime( qso_record.date_adif, 9, "%Y%m%d",   &dt );
  strftime( qso_record.time_adif, 5, "%H%M",     &dt );

  /* Enter mode of operation */
  if( isFlagSet(MODE_BPSK) )
	strcpy( qso_record.mode, "BPSK" );
  else
	strcpy( qso_record.mode, "QPSK" );
}


void
on_save_record_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  /* Save record if needed (and valid) */
  if( Read_QSO_Record() )
	Save_Dialog( "Save QSO Record to file?" );
  else
	Save_Dialog(
		"QSO Record is incomplete\n"
		"Do you still want to save?" );
}


void
on_clear_fields_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Clear_Record_Fields(TRUE);
}


void
on_error_quit_button_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Cleanup();
  gtk_main_quit();
}


void
on_cancel_save_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  gtk_widget_destroy( save_dialog );
}


void
on_save_ok_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  gtk_widget_destroy( save_dialog );
  Save_QSO_Record();
}

void
on_bpsk_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_BPSK );
	ClearFlag( MODE_QPSK_USB | MODE_QPSK_LSB );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
  }
}


void
on_qpsk_lsb_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_QPSK_LSB );
	ClearFlag( MODE_QPSK_USB |
		MODE_BPSK );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
	Init_Viterbi();
  }
}


void
on_qpsk_usb_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( MODE_QPSK_USB );
	ClearFlag( MODE_QPSK_LSB |
		MODE_BPSK );
	Set_Tcvr_Mode();
	Set_TxRx_Labels();
	Init_Viterbi();
  }
}


void
on_capitalize_letters_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
	SetFlag( CAPITAL_LETTERS );
  else
	ClearFlag( CAPITAL_LETTERS );
}


void
on_quit_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( isFlagSet(TRANSMIT_MODE) ||
	  isFlagSet(RECEIVE_MODE) )
	Quit_Dialog(
		"xpsk31 is running\n"
		"Still want to quit?" );
  else
	gtk_main_quit();
}


void
on_record_qsos_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  /* File path for QSO record  */
  char qso_record_fpath[64];

  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	/* Make file path for record file */
	snprintf( qso_record_fpath, 64,
		"%s/xpsk31/record.txt", getenv("HOME") );

	/* Create a file for recording QSO's */
	if( qso_record_fp == NULL )
	{
	  qso_record_fp = fopen( qso_record_fpath, "a" );
	  if( qso_record_fp == NULL )
	  {
		perror( qso_record_fpath );
		Error_Dialog(
			"Failed to open QSO Record file\n"
			"Quit xpsk31 and correct", FALSE );
		return;
	  }
	} /* if( QSO_record_fp == NULL ) */

	SetFlag( RECORD_QSO );

  } /* if( gtk_check_menu_item_get_active( */
  else
  {
	if( qso_record_fp != NULL )
	{
	  fclose( qso_record_fp );
	  qso_record_fp = NULL;
	  ClearFlag( RECORD_QSO );
	}

  } /* else */
}

void
on_identify_in_cw_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  char message[24];

  snprintf( message, 24, "  DE %s  TU", rc_data.call );
  Morse_Transmit( message );
}


void
on_bad_entry_ok_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  gtk_widget_destroy( bad_entry_dialog );
}


void
on_cancel_quit_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  gtk_widget_destroy( quit_dialog );
}


void
on_quit_ok_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Cleanup();
  gtk_main_quit();
}


void
on_clear_window_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  gtk_text_buffer_set_text( text_buffer, "", -1 );
}


gboolean
on_tx_textview_key_press_event(
	GtkWidget       *widget,
	GdkEventKey     *event,
	gpointer         user_data)
{
  if( (event->keyval == GDK_Shift_L) ||
	  (event->keyval == GDK_Shift_R) )
	return FALSE;

  key_buff[key_cnt] = event->keyval;

  if( ++key_cnt == 32 )
	key_cnt = 0;

  return TRUE;
}

gboolean
on_scope_expose_event(
	GtkWidget       *widget,
	GdkEventExpose  *event,
	gpointer         user_data)
{
  if( isFlagSet(ENABLE_MAGNIPHASE) )
	gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Magniphase" );
  else
  {
	/* Draw waterfall */
	gdk_draw_pixbuf(
		widget->window, NULL, gbl_wfall_pixbuf,
		event->area.x, event->area.y,
		event->area.x, event->area.y,
		event->area.width, event->area.height,
		GDK_RGB_DITHER_NONE, 0, 0 );
	gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Waterfall" );
  }

  return TRUE;
}


gboolean
on_scope_button_press_event(
	GtkWidget       *widget,
	GdkEventButton  *event,
	gpointer         user_data)
{
  switch( event->button )
  {
	case 3:
	  if( isFlagSet( ENABLE_MAGNIPHASE ) )
	  {
		gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Waterfall" );
		ClearFlag( ENABLE_MAGNIPHASE );
	  }
	  else
	  {
		gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Magniphase" );
		SetFlag( ENABLE_MAGNIPHASE );
	  }
	  gtk_widget_queue_draw( widget );
	  break;

	case 2:
	  if( isFlagSet( REVERSE_VIDEO ) )
		ClearFlag( REVERSE_VIDEO );
	  else
		SetFlag( REVERSE_VIDEO );
	  break;

	case 1:
	  if( isFlagSet(ENABLE_CAT)   &&
		  isFlagSet(RECEIVE_MODE) &&
		  isFlagClear(ENABLE_MAGNIPHASE) )
		Tune_Tcvr( event->x );

  } /* switch( event->button ) */

  return TRUE;
}


void
on_clear_windows_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  gtk_text_buffer_set_text(gbl_tx_text_buffer, "", -1);
  gtk_text_buffer_set_text(gbl_rx_text_buffer, "", -1);
}


void
on_freq_adjust_value_changed(
	GtkRange        *range,
	gpointer         user_data)
{
  GtkWidget *hscale;
  GtkAdjustment *adjustment;

  /* Read and enter freq correction */
  hscale = lookup_widget( main_window, "freq_adjust" );
  adjustment = gtk_range_get_adjustment( GTK_RANGE(hscale) );
  freq_error = (int)(adjustment->value*10.);
}


void
on_ft847_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( ENABLE_CAT_847 );
	ClearFlag( ENABLE_CAT_857 );
	ClearFlag( ENABLE_RTS_CTL );
	Close_Tcvr_Serial();
	Open_Tcvr_Serial();
  }
}


void
on_ft857_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( ENABLE_CAT_857 );
	ClearFlag( ENABLE_CAT_847 );
	ClearFlag( ENABLE_RTS_CTL );
	Close_Tcvr_Serial();
	Open_Tcvr_Serial();
  }
}


void
on_rts_ctlt_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
  {
	SetFlag( ENABLE_RTS_CTL );
	ClearFlag( ENABLE_CAT_847 );
	ClearFlag( ENABLE_CAT_857 );
	Close_Tcvr_Serial();
	Open_RTS_Serial();
  }
}


void
on_none_activate(
	GtkMenuItem     *menuitem,
	gpointer         user_data)
{
  if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
	Close_Tcvr_Serial();
}


void
on_xmit_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  /* Toggle Tx/Rx/Macro modes */
  if( isFlagSet(TRANSMIT_MACRO) )
	ClearFlag( TRANSMIT_MACRO  );
  else
	if( isFlagSet(TRANSMIT_KEYBD) )
	{
	  ClearFlag( TRANSMIT_KEYBD  );
	  SetFlag( RECEIVE_MODE );
	  gtk_idle_add( Receive_Mode, NULL );
	}
	else
	{
	  SetFlag( TRANSMIT_KEYBD );
	  ClearFlag( RECEIVE_MODE );
	  Set_TxRx_Labels();
	  gtk_idle_add( Transmit_Keybd, "dummy" );
	}

  return;
}


void
on_rcve_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  /* Toggle Tx/Rx/Macro modes */
  if( isFlagSet(RECEIVE_MODE) )
  {
	ClearFlag( RECEIVE_MODE );
	Set_TxRx_Labels();
  }
  else
  {
	if( isFlagClear(TRANSMIT_MACRO) )
	{
	  SetFlag( RECEIVE_MODE );
	  ClearFlag( TRANSMIT_KEYBD );
	  gtk_idle_add( Receive_Mode, NULL );
	}
	else
	{
	  ClearFlag( TRANSMIT_MACRO );
	  SetFlag( TRANSMIT_KEYBD );
	  Set_TxRx_Labels();
	}

  } /* if( isFlagSet(RECEIVE_MODE) ) */

  return;
}


void
on_rst_out_changed(
	GtkEditable     *editable,
	gpointer         user_data)
{
  Process_RST( editable );
  strcpy( qso_record.dx_rst, gtk_entry_get_text(GTK_ENTRY(editable)) );
}


void
on_rst_in_changed(
	GtkEditable     *editable,
	gpointer         user_data)
{
  Process_RST( editable );
  strcpy( qso_record.my_rst, gtk_entry_get_text(GTK_ENTRY(editable)) );
}


void
on_macro1_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 1 );
}


void
on_macro2_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 2 );
}


void
on_macro3_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 3 );
}


void
on_macro4_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 4 );
}


void
on_macro5_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 5 );
}


void
on_macro6_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 6 );
}


void
on_macro7_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 7 );
}


void
on_macro8_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 8 );
}


void
on_macro9_clicked(
	GtkButton       *button,
	gpointer         user_data)
{
  Select_Macro( 9 );
}



void
on_afc_toggled(
	GtkToggleButton *togglebutton,
	gpointer         user_data)
{
  if( gtk_toggle_button_get_active(togglebutton) )
	SetFlag( ENABLE_AFC );
  else
  {
	ClearFlag( ENABLE_AFC );
	freq_error = 0;
  }
}


void
on_error_dialog_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
  error_dialog = NULL;
}


void
on_error_ok_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy( error_dialog );
}


gboolean
on_error_dialog_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return TRUE;
}


gboolean
on_bad_entry_dialog_delete_event       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return TRUE;
}


void
on_scope_size_allocate                 (GtkWidget       *widget,
                                        GdkRectangle    *allocation,
                                        gpointer         user_data)
{
  /* Destroy existing pixbuff */
  if( gbl_wfall_pixbuf != NULL )
  {
	g_object_unref( G_OBJECT(gbl_wfall_pixbuf) );
	gbl_wfall_pixbuf = NULL;
  }

  /* Create waterfall pixbuf */
  gbl_wfall_pixbuf = gdk_pixbuf_new(
	  GDK_COLORSPACE_RGB, FALSE, 8,
	  allocation->width, allocation->height );
  if( gbl_wfall_pixbuf == NULL ) return;

  gbl_wfall_pixels = gdk_pixbuf_get_pixels( gbl_wfall_pixbuf );
  gbl_wfall_width  = gdk_pixbuf_get_width ( gbl_wfall_pixbuf );
  gbl_wfall_height = gdk_pixbuf_get_height( gbl_wfall_pixbuf );
  gbl_wfall_rowstride  = gdk_pixbuf_get_rowstride( gbl_wfall_pixbuf );
  gbl_wfall_n_channels = gdk_pixbuf_get_n_channels( gbl_wfall_pixbuf );
  Clear_Wfall_Pixbuf();
}

